const Crisp = require("crisp-api");
const CrispClient = new Crisp();
const config = require('config');

const identifier = config.get('credenciales.crisp.identifier');
const key = config.get('credenciales.crisp.key');
const SIGUIENTE_PASO = 'siguiente_paso';
const DELAY_MS = 1000;

let localData = {};

CrispClient.authenticateTier("plugin", identifier, key);

async function sendTextMessage(website_id, session_id, message) {
    await composeMessage(website_id, session_id, DELAY_MS);
    await CrispClient.website.sendMessageInConversation(
        website_id,
        session_id, {
            type: "text",
            from: "operator",
            origin: "chat",
            content: message
        }
    );
}

async function composeMessage(website_id, session_id, ms){
    await CrispClient.website.composeMessageInConversation(
        website_id,
        session_id, {
            "type": "start",
            "from": "operator"}
    );
    await new Promise(resolve => setTimeout(resolve, ms));
    await CrispClient.website.composeMessageInConversation(
        website_id,
        session_id, {
            "type": "stop",
            "from": "operator"}
    );
}

async function showWritingIcon(website_id, session_id){
    await composeMessage(website_id, session_id, DELAY_MS);
}

async function sendPickerMessage(website_id, session_id, id, title, choices) {
    await composeMessage(website_id, session_id, DELAY_MS);
    await CrispClient.website.sendMessageInConversation(
        website_id,
        session_id, {
            type: "picker",
            from: "operator",
            origin: "chat",
            content: {
                "id": id,
                "text": title,
                "choices": choices //array
            }
        }
    );
}

async function getSiguientePaso(website_id, session_id) {
    let data = await getUserData(website_id, session_id);
    return data[SIGUIENTE_PASO];
}

async function setSiguientePaso(website_id, session_id, siguientePaso, userData) {
    if (userData == null) {
        userData = await getUserData(website_id, session_id);
    }
    userData[SIGUIENTE_PASO] = siguientePaso;
    return await updateUserData(website_id, session_id, userData);
}

async function getUserData(website_id, session_id) {
    let data = localData[website_id.concat(session_id)];
    if(data == null){
        await updateUserData(website_id, session_id, {});
        return getUserData(website_id, session_id);
    }
    return data;
}

async function updateCrispData(website_id, session_id, correo, tipo_documento, numero_documento, fecha_nacimiento, sexo) {
    await CrispClient.website.updateConversationMetas(website_id, session_id, {
        email: correo,
        data: {
            tipo_documento: tipo_documento,
            numero_documento: numero_documento,
            fecha_nacimiento: fecha_nacimiento,
            sexo: sexo,
        }
    });
}

async function getCrispData(website_id, session_id) {
    return await CrispClient.website.getConversationMetas(website_id, session_id);
}

async function updateUserData(website_id, session_id, data) {
    localData[website_id.concat(session_id)] = data;
}

module.exports =
    {
        CrispClient: CrispClient,
        showWritingIcon: showWritingIcon,
        sendTextMessage: sendTextMessage,
        sendPickerMessage: sendPickerMessage,
        getSiguientePaso: getSiguientePaso,
        setSiguientePaso: setSiguientePaso,
        getUserData: getUserData,
        updateUserData: updateUserData,
        updateCrispData: updateCrispData,
        getCrispData: getCrispData
    }